<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:dbk="http://docbook.org/ns/docbook"
  xmlns:tr="http://transpect.io"
  xmlns:idml2xml="http://transpect.io/idml2xml"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:css="http://www.w3.org/1996/css"
  xmlns:hub="http://transpect.io/hub"
  xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main"
  xmlns:docx2hub="http://transpect.io/docx2hub"
  xmlns:mml="http://www.w3.org/1998/Math/MathML"
  xmlns:functx="http://www.functx.com"
  xmlns:cx="http://xmlcalabash.com/ns/extensions"
  xmlns="http://docbook.org/ns/docbook"
  xpath-default-namespace="http://docbook.org/ns/docbook"
  exclude-result-prefixes="xs tr xlink hub w docx2hub dbk idml2xml mml functx cx"
  version="2.0">
  
	<xsl:import href="http://transpect.io/evolve-hub/xsl/evolve-hub.xsl"/>
  <xsl:import href="http://transpect.io/xslt-util/functx/xsl/functx.xsl"/>
  
  <xsl:param name="hub:hierarchy-title-roles" as="xs:boolean" select="true()"/>
  <xsl:variable name="hub:hierarchy-role-regexes-x" as="xs:string+"
    select="(
             'berschrift1$|^Heading1$|^heading_1$',
             'berschrift2$|^Heading2$|^heading_2$',
             'berschrift3$|^Heading3$|^heading_3$',
             'berschrift4$|^Heading4$|^heading_4$',
             'berschrift5$|^Heading5$|^heading_5$',
             'berschrift6$|^Heading6$|^heading_6$')" />
  
	<xsl:variable name="hub:handle-several-images-per-caption" as="xs:boolean" select="true()"/>
  <xsl:variable name="hub:figure-caption-must-begin-with-figure-caption-start-regex"  as="xs:boolean"
    select="false()" />
 
  <xsl:variable name="hub:remove-para-wrapper-for-mediaobject"  as="xs:boolean"
    select="true()" />
  
  <xsl:variable name="hub:thead-pstyle-regex" as="xs:string"
    select="'^Tableheader(_?merged)?'" />
  
   <xsl:variable name="hub:itemizedlist-mark-chars-regex" as="xs:string"
    select="'([&#x6f;&#xb7;&#x25aa;&#x25a1;&#x25b6;&#x25cf;&#x2212;&#x2022;\p{So}\p{Pd}&#x23af;&#xF0B7;&#xF0BE;&#61485;-])'"/>
  
  <xsl:function name="hub:to-twips" as="xs:double">
    <xsl:param name="css-length" as="xs:string"/>
    <xsl:choose>
      <xsl:when test="ends-with($css-length, 'pt')">
        <xsl:sequence select="xs:double(replace($css-length, 'pt$', '')) * 20" />
      </xsl:when>
      <xsl:when test="matches($css-length, 'auto')">
        <xsl:sequence select="0" />
      </xsl:when>
      <xsl:otherwise>
        <!-- provisional: -->
        <xsl:sequence select="number($css-length)" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
    
  <xsl:template match="annotation
                      | anchor[matches(@xml:id,'^comment')]" mode="hub:split-at-tab"/>
  
   <xsl:template match="anchor[matches(@role,'end')]
                       | anchor[matches(@xml:id,'^page_')] " mode="hub:split-at-tab" />

  <!--  discard anchors without link to it -->
  <xsl:template match="anchor[matches(@role, 'start')]
                             [not(key('hub:linking-item-by-id', @xml:id))]" mode="hub:split-at-tab" priority="3"/>
 
 
  <xsl:template match="blockquote[matches(@role, '^hub:')]
                      | phrase[@role='CommentReference']" mode="hub:clean-hub">
   <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="blockquote[parent::blockquote][every $b in * satisfies $b[self::blockquote[matches(@role, parent::blockquote/@role)]]]" mode="hub:clean-hub">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="phrase[mediaobject]" mode="hub:clean-hub">
    <xsl:apply-templates select="mediaobject" mode="#current"/>
   </xsl:template>
  
  <xsl:template match="para[not(node())]
                       | para[matches(@role,'^Titlepage$')]" mode="hub:clean-hub"/>
  
  <xsl:template match="title[descendant::anchor]" mode="hub:postprocess-hierarchy">
    <xsl:copy>
      <xsl:attribute name="xml:id" select="descendant::anchor[1]/@xml:id"/>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="*[para[matches(@role, '[Nn]ote')]]" mode="hub:postprocess-lists">
    <xsl:copy>
      <xsl:for-each-group select="node()" group-adjacent="string(@role)">
        <xsl:choose>
          <xsl:when test="matches(current-grouping-key(), '[Nn]ote')">
            <xsl:element name="blockquote">
              <xsl:attribute name="role" select="'note'"/>
              <xsl:apply-templates select="current-group()"/>
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="current-group()"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each-group>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="phrase[@role='hub:foreign'][descendant::*:webVideoPr]" mode="hub:split-at-tab">
    <xsl:copy>
      <xsl:attribute name="role" select="'video'"/>
      <xsl:value-of select="descendant::*:webVideoPr/@embeddedHtml"/>
    </xsl:copy>
  </xsl:template>
  
  <!-- fix itemizedlist with o mark  -->
  <xsl:template match="itemizedlist/@mark[.='loweralpha']" mode="hub:postprocess-lists">
    <xsl:attribute name="mark" select="'bullet'"/>
  </xsl:template>
  
<!--  <xsl:template match="section[not(ancestor::section)]" mode="hub:ids">
    <xsl:param name="parent_id"/>
    <xsl:copy>
      <xsl:attribute name="xml:id" select="concat('s',count(preceding-sibling::section)+1)"/>  
      <xsl:apply-templates select="@* except @xml:id, node()" mode="#current">
        <xsl:with-param name="parent_id" select="concat('s',count(preceding-sibling::section)+1)"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="section" mode="hub:ids">
    <xsl:param name="parent_id"/>
    <xsl:copy>
      <xsl:attribute name="xml:id" select="concat($parent_id,'.',count(preceding-sibling::section)+1)"/>
      <xsl:apply-templates select="@* except @xml:id, node()" mode="#current">
        <xsl:with-param name="parent_id" select="concat($parent_id,'.',count(preceding-sibling::section)+1)"/>
      </xsl:apply-templates>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="informaltable" mode="hub:ids">
    <xsl:copy>
      <xsl:attribute name="xml:id" select="concat('tab', count(preceding::informaltable)+1)"/>
      <xsl:apply-templates select="@* except @xml:id, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>-->
  
  <xsl:template match="phrase[equation]" mode="hub:clean-hub">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
</xsl:stylesheet>