<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:jats="http://jats.nlm.nih.gov" 
  xmlns:dbk="http://docbook.org/ns/docbook" 
  xmlns:css="http://www.w3.org/1996/css"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:hub="http://transpect.io/hub"
  xmlns:functx="http://www.functx.com"
  xmlns:mml="http://www.w3.org/1998/Math/MathML"
  xmlns:tr= "http://transpect.io"
  xmlns:docx2hub="http://transpect.io/docx2hub"
  xmlns:cx="http://xmlcalabash.com/ns/extensions"
  exclude-result-prefixes="cx css jats functx hub dbk xs tr mml docx2hub" version="2.0">

  <xsl:import href="http://transpect.io/hub2bits/xsl/hub2jats-fallback.xsl"/>
  <xsl:import href="http://transpect.io/xslt-util/functx/xsl/functx.xsl"/>
  <xsl:import href="http://transpect.io/xslt-util/hex/xsl/hex.xsl"/>
  <xsl:template name="custom-meta-group"/>
  
  <xsl:output method="xml" indent="no"/>
  
  <xsl:variable name="source-basename" select="/dbk:hub/dbk:info/dbk:keywordset/dbk:keyword[matches(@role, 'source-basename')]"/>
  
  <xsl:variable name="css:wrap-content-with-elements-from-mappable-style-attributes" as="xs:boolean"
    select="false()"/>

  <xsl:variable name="dtd-version-att" as="attribute(dtd-version)">
    <xsl:attribute name="dtd-version" select="'1.1'" />
  </xsl:variable>
  

  <xsl:template name="meta">
     <journal-meta>
      <journal-id/>
      <issn/>
    </journal-meta>
    <article-meta>
      <article-id><xsl:value-of select="$source-basename"/></article-id>
      <!-- article categorie/ series created from naming convention/cascade? -->
      <article-categories/>
      <title-group>
        <article-title><xsl:value-of select="following-sibling::dbk:para[not(ancestor::section)][matches(@role,'title|Titel')]/descendant::text()"/></article-title>
        <xsl:if test="following-sibling::dbk:para[not(ancestor::section)][not(matches(@rol,'title|Titel'))]">
          <subtitle><xsl:value-of  select="following-sibling::dbk:para[not(ancestor::section)][not(matches(@role,'title|Titel'))]/descendant::text()"/></subtitle>
        </xsl:if>
      </title-group>
      <permissions/>
      <custom-meta-group>
        <custom-meta>
          <meta-name>markup-level</meta-name>
          <meta-value>base</meta-value>
        </custom-meta>
      </custom-meta-group>
    </article-meta>
  </xsl:template>
  
  <xsl:template match="dbk:para[not(ancestor::section) and preceding-sibling::*[self::dbk:info]]" mode="default"/>
  
 <xsl:template match="custom-meta-group | css:rules" mode="clean-up"/>
  
  <xsl:template match="dbk:entry/dbk:para[preceding-sibling::*[1]/self::dbk:para]" priority="7" mode="default">
    <break/>
    <xsl:next-match/>
  </xsl:template>

  <xsl:template match="mml:math" mode="default">
    <xsl:copy-of select="." copy-namespaces="no"/>
  </xsl:template>
  
  <xsl:template match="dbk:equation" mode="default">
    <disp-formula>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </disp-formula>
  </xsl:template>
  
  <xsl:template match="styled-content" mode="clean-up">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
   
  <xsl:template match="sub/@style-type 
                      | sup/@style-type 
                      | *[matches(local-name(),'bold|italic|underline')]/@style-type
                      | dbk:annotation
                      | @disp-level 
                      | @hub:anchored
                      | @css:*
                      | *:target
                      |*[not(self::*[local-name()=('xref','col','break', 'td', 'graphic', 'th', 'ext-link', 'issn', 'journal-id', 'inline-graphic')])]
                         [not(node())]
                      | list-item/label" mode="clean-up"/>
  
  
  <xsl:template match="@docx2hub:*" mode="default"/>
  
  <xsl:template match="dbk:indexterm/@type" mode="default">
    <xsl:attribute name="index-type" select="."/>
  </xsl:template>
  
  <xsl:template match="break[not(ancestor::*[matches(local-name(),'th|td|title|subtitle|disp-formula|named-content')])]
                      |break[ancestor::list and ancestor::table]" mode="clean-up"/>
  
  <xsl:template match="dbk:table/@frame | dbk:informaltable/@frame" mode="default" priority="2">
    <xsl:variable name="type" as="xs:string?">
      <xsl:choose>
        <xsl:when test=". = 'all'">
          <xsl:value-of select="'box'"/>
        </xsl:when>
        <xsl:when test=". = 'top'">
          <xsl:value-of select="'above'"/>
        </xsl:when>
        <xsl:when test=". = 'bottom'">
          <xsl:value-of select="'below'"/>
        </xsl:when>
        <xsl:when test=". = 'topbot'">
          <xsl:value-of select="'hsides'"/>
        </xsl:when>
        <xsl:when test=". = 'none'">
          <xsl:value-of select="'void'"/>
        </xsl:when>
        <xsl:when test=". = 'sides'">
          <xsl:value-of select="'vsides'"/>
        </xsl:when>
      </xsl:choose>
    </xsl:variable>
    <xsl:if test="$type">
      <xsl:attribute name="frame" select="$type"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="table-wrap/@frame" mode="clean-up"/>
  
  <xsl:template match="table[parent::table-wrap/@frame]" mode="clean-up">
    <xsl:copy>
      <xsl:attribute name="frame" select="parent::table-wrap/@frame"/>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="p[styled-content[@style-type='hub:line']]
                        [every $n in * satisfies $n[self::styled-content[@style-type='hub:line'] or self::break]]" mode="clean-up">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="styled-content[@style-type='hub:line'][parent::p]" mode="clean-up" priority="2">
   <p>
     <xsl:if test="parent::p/@content-type">
       <xsl:attribute name="content-type" select="parent::p/@content-type"/>
     </xsl:if>
     <xsl:apply-templates select="@srcpath, node()" mode="#current"/>
   </p>
 </xsl:template>
  
  <xsl:template match="fn/p[@content-type='Fussnote']
                           [styled-content[@style-type='hub:line']]                           
                           [every $n in * satisfies $n[self::styled-content[@style-type='hub:identifier'] or self::styled-content[@style-type='hub:line'] or self::break]]" mode="clean-up">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>
  
  <xsl:template match="fn[p[@content-type='Fussnote']
                           [styled-content[@style-type='hub:identifier']]]" mode="clean-up">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <label>
        <xsl:value-of select="p/styled-content[@style-type='hub:identifier']"/>
      </label>
      <xsl:apply-templates select="node() except p/styled-content[@style-type='Fussnotenziffer']" mode="#current"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="styled-content[@style-type='Fussnotenziffer'][ancestor::fn]" mode="clean-up"/>
  
</xsl:stylesheet>
