<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:css="http://www.w3.org/1996/css"
  xmlns:dbk="http://docbook.org/ns/docbook"
  xmlns:hub="http://transpect.io/hub"
  xmlns:hub2tei="http://transpect.io/hub2tei"
  xmlns:tei="http://www.tei-c.org/ns/1.0"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:html="http://www.w3.org/1999/xhtml"
  xmlns:tr="http://transpect.io"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:cx="http://xmlcalabash.com/ns/extensions"
  xmlns="http://www.tei-c.org/ns/1.0"
  exclude-result-prefixes="dbk hub2tei hub xlink css xs tr cx tei"
  version="2.0">

  <xsl:import href="http://transpect.io/hub2tei/xsl/hub2tei.xsl"/>

  <xsl:output name="debug" method="xml" indent="no"/>

	<xsl:template match="css:rules" mode="hub2tei:dbk2tei"/>

  <xsl:template match="/dbk:*" mode="hub2tei:dbk2tei">
    <TEI>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:attribute name="source-dir-uri" select="dbk:info/dbk:keywordset[@role eq 'hub']/dbk:keyword[@role eq 'source-dir-uri']"/>
      <teiHeader>
        <fileDesc>
          <titleStmt>
            	<xsl:call-template name="title-stm"/>
          </titleStmt>
         <editionStmt>
            	<xsl:call-template name="edition-stm"/>
           </editionStmt>
          <publicationStmt>
            <xsl:call-template name="publication-stm"/>
          </publicationStmt>
          <sourceDesc>
          	<xsl:call-template name="source-desc"/>
          </sourceDesc>
        </fileDesc>
        <profileDesc>
          <textClass>
           <xsl:apply-templates select="dbk:info/dbk:keywordset[@role = 'hub']" mode="#current"/>
           <xsl:apply-templates select="dbk:info/dbk:keywordset[not(@role = 'hub')]" mode="#current"/>
          </textClass>
          <xsl:if test="dbk:info/dbk:abstract">
            <abstract>
              <xsl:apply-templates select="dbk:info/dbk:abstract/node()" mode="#current"/>
            </abstract>
          </xsl:if>
          <langUsage>
            <language>
              <xsl:attribute name="ident">
                <xsl:apply-templates select="/dbk:*/@xml:lang" mode="#current"/>
              </xsl:attribute>
            </language>
          </langUsage>
        </profileDesc>
        <encodingDesc>
          <styleDefDecl scheme="cssa"/>
          <xsl:apply-templates select="/*/dbk:info/css:rules" mode="#current"/>
        </encodingDesc>
      </teiHeader>
      <text>
        <xsl:apply-templates select="dbk:info" mode="#current"/>
        <xsl:variable name="backmatter" as="element(*)*"
          select="(., .[count(dbk:part) = 1]/dbk:part)/(dbk:appendix, dbk:glossary, dbk:bibliography, dbk:index)"/>
        <!-- TO DO: include and respect exclude param in future template that processes dbk:bibliography -->
      	<body>
      		<xsl:variable name="body" as="element(*)*">
      			<xsl:apply-templates select="* except dbk:info" mode="#current">
      				<xsl:with-param name="exclude" select="$backmatter" tunnel="yes"/>
      			</xsl:apply-templates>
      		</xsl:variable>
      		<xsl:choose>
      			<xsl:when test="$body[node()]">
      				<xsl:choose>
      				  <xsl:when test="exists($body[self::*[local-name() = ('p', 'div', 'ab', 'bibl', 'biblStruct', 'castList', 'classSpec', 'constraintSpec',
      				    'desc', 'div', 'div1', 'divGen', 'eTree', 'eg', 'elementSpec', 'entry', 'entryFree', 'floatingText',
      				    'forest', 'graph ', 'l', 'label', 'lg', 'list', 'listApp', 'listBibl', 'listEvent', 'listForest',
      				    'listNym', 'listOrg', 'listPerson', 'listPlace', 'listRef', 'listTranspose', 'listWit', 'macroSpec',
      				    'moduleSpec', 'move', 'msDesc', 'q', 'quote', 'said', 'schemaSpec', 'sound', 'sp', 'spGrp', 'specGrp',
      				    'specGrpRef', 'stage', 'superEntry', 'table', 'tech', 'tree', 'u', 'view')]])">
      						<xsl:sequence select="$body"/>
      					</xsl:when>
      					<xsl:otherwise>
      						<!-- if a body contains neither paras nor divs etc., it is a schema error (case: only sidebars in image parts)-->
      						<div type="chapter" rend="virtual">
      							<xsl:sequence select="$body"/>
      						</div>
      					</xsl:otherwise>
      				</xsl:choose>
      			</xsl:when>
      			<xsl:otherwise>
      				<p srcpath="{generate-id()}"/>
      			</xsl:otherwise>
      		</xsl:choose>
      	</body>
        <xsl:if test="exists($backmatter)">
          <back>
            <xsl:apply-templates select="$backmatter" mode="#current"/>
          </back>
        </xsl:if>
      </text>
    </TEI>
  </xsl:template>

  <xsl:template name="title-stm">
    <title>
      <xsl:value-of select="(dbk:para[matches(@role , '[tT]it[el|le]')][1],dbk:section[1]/dbk:title)[1]"/>
    </title>
    <author>
      <xsl:value-of select="dbk:info/dbk:keywordset[@role eq 'docProps']/dbk:keyword[@role eq 'dc:creator']"/>
    </author>
  </xsl:template>

  <xsl:template name="edition-stm">
    <edition>
      <date>
        <xsl:value-of select="replace(dbk:info/dbk:keywordset[@role eq 'docProps']/dbk:keyword[@role eq 'dcterms:created'], 'T.*','')"/>
      </date>
    </edition>
  </xsl:template>

  <xsl:template name="source-desc">
    <p>Converted from a Word document</p>
  </xsl:template>

  <xsl:template name="publication-stm">
    <p>unknown</p>
  </xsl:template>

  <xsl:template match="@renderas" mode="hub2tei:dbk2tei"/>

	<xsl:template match="dbk:info/dbk:keywordset" mode="hub2tei:dbk2tei"/>

  <xsl:template match="dbk:phrase | dbk:sub | dbk:sup" mode="hub2tei:dbk2tei">
    <hi>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </hi>
  </xsl:template>

  <!--<xsl:template match="dbk:listitem | dbk:glossdef" mode="hub2tei:dbk2tei">
    <item rend="{if (parent::*[self::dbk:itemizedlist]) then parent::*[self::dbk:itemizedlist]/@mark else parent::*[self::dbk:orderedlist]/@numeration}"
          n="{position()}">
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </item>
  </xsl:template>-->

 <!-- <xsl:template match="@rend" mode="hub2tei:tidy">
    <xsl:attribute name="rend" select="replace(., '_', ' ')"/>
	</xsl:template>-->

  <!--	adaptations from wordtei2gsktei.xsl -->

  <!-- lead paragraph -->
  <xsl:template match="tei:p[@rend = 'Lead']/@rend" mode="hub2tei:tidy" priority="2">
    <xsl:attribute name="rend">lead</xsl:attribute>
  </xsl:template>

  <!-- superscript to sup -->
  <xsl:template match="tei:hi[@rend = 'superscript']/@rend" mode="hub2tei:tidy" priority="2">
    <xsl:attribute name="rend">sup</xsl:attribute>
  </xsl:template>

  <!-- image [todo: string auseinandernehmen] -->
  <xsl:template match="tei:p[@rend = 'Bildeinfuegen']" mode="hub2tei:tidy">
    <figure xmlns="http://www.tei-c.org/ns/1.0">
      <graphic>
        <xsl:analyze-string select="." regex="\[(\w+)='?([^\]]+)'?\]" flags="m">
          <xsl:matching-substring>
            <xsl:choose>
              <xsl:when test="regex-group(1) = ('width', 'height', 'rend')">
                <xsl:attribute name="{regex-group(1)}">
                  <xsl:value-of select="regex-group(2)"/>
                </xsl:attribute>
              </xsl:when>
            </xsl:choose>
          </xsl:matching-substring>
          <xsl:non-matching-substring>
            <xsl:if test="normalize-space(.) != ''">
              <xsl:attribute name="url">
                <xsl:value-of select="replace(normalize-space(.), '^(.*)\.[^\.]+$', '$1')"/>
              </xsl:attribute>
            </xsl:if>
          </xsl:non-matching-substring>
        </xsl:analyze-string>
      </graphic>
      <xsl:analyze-string select="." regex="\[([^=]+)='?([^'\]]+)'?\]" flags="m">
        <xsl:matching-substring>
          <xsl:if test="regex-group(1) = 'figDesc'">
            <figDesc>
              <xsl:value-of select="regex-group(2)"/>
            </figDesc>
          </xsl:if>
        </xsl:matching-substring>
      </xsl:analyze-string>
    </figure>
  </xsl:template>

  <!-- orgName -->
  <xsl:template match="tei:hi[@rend = 'Organisation']" mode="hub2tei:tidy">
    <orgName xmlns="http://www.tei-c.org/ns/1.0">
      <xsl:apply-templates select="node()"/>
    </orgName>
  </xsl:template>

  <!--persName -->
  <xsl:template match="tei:hi[@rend = 'Person']" mode="hub2tei:tidy">
    <persName key="{following-sibling::*[@rend='annotation_reference'][1]/tei:note[1]/descendant::text()}"
      xmlns="http://www.tei-c.org/ns/1.0">
      <xsl:apply-templates select="node()"/>
    </persName>
  </xsl:template>

  <!-- placeName -->
  <xsl:template match="tei:hi[@rend = 'PlaceName']" mode="hub2tei:tidy">
    <placeName
      key="{following-sibling::*[@rend='annotation_reference'][1]/tei:note[1]/descendant::text()}"
      xmlns="http://www.tei-c.org/ns/1.0">
      <xsl:apply-templates select="node()"/>
    </placeName>
  </xsl:template>

  <!-- heads with comment -->
  <xsl:template match="tei:head[tei:hi/@rend = 'annotation_reference']" mode="hub2tei:tidy">
    <head xmlns="http://www.tei-c.org/ns/1.0"><placeName
      key="{descendant::*[@rend='annotation_reference'][1]/tei:note[1]/descendant::text()}">
      <xsl:apply-templates select="node()"/>
    </placeName>
    </head>
  </xsl:template>

  <xsl:template match="tei:persName[@key = '']" mode="hub2tei:tidy"/>
  <xsl:template match="tei:placeName[@key = '']" mode="hub2tei:tidy"/>

  <!-- Abb Verweis -->
  <xsl:template match="tei:hi[@rend = 'Abb_verweis']" mode="hub2tei:tidy">
    <ref type="img" xmlns="http://www.tei-c.org/ns/1.0">
      <xsl:apply-templates select="node()"/>
    </ref>
  </xsl:template>

  <!-- Endnotes -->
  <xsl:template match="tei:hi[@rend = 'endnote_reference']" mode="hub2tei:tidy">
    <ref xmlns="http://www.tei-c.org/ns/1.0">
      <xsl:apply-templates select="node()"/>
    </ref>
  </xsl:template>

	<xsl:template match="dbk:blockquote" mode="hub2tei:dbk2tei">
		<quote rendition="block">
			<xsl:apply-templates select="node()" mode="#current"/>
		</quote>
	</xsl:template>

	<xsl:template match="dbk:quote" mode="hub2tei:dbk2tei">
		<quote rendition="inline">
			<xsl:apply-templates select="node()" mode="#current"/>
		</quote>
	</xsl:template>

	<xsl:template match="dbk:bibliomixed | dbk:para[@role = 'GFN-BibliografischerEintrag']" priority="3" mode="hub2tei:dbk2tei">
		<bibl>
			<xsl:apply-templates select="@*" mode="#current"/>
			<xsl:attribute name="xml:id" select="generate-id()"/>
			<xsl:apply-templates select="node()" mode="#current"/>
		</bibl>
	</xsl:template>

	<xsl:template match="dbk:footnote" mode="hub2tei:dbk2tei">
		<note n="{@xreflabel}">
			<xsl:apply-templates select="@xml:id" mode="#current"/>
			<xsl:apply-templates select="node()" mode="#current"/>
		</note>
	</xsl:template>

 <xsl:template match="dbk:equation|dbk:inlineequation" mode="hub2tei:dbk2tei">
		<formula notation="mathml">
			<xsl:apply-templates select="node()" mode="#current"/>
		</formula>
	</xsl:template>
  
  <xsl:template match="dbk:inlineequation" mode="hub2tei:dbk2tei">
		<formula notation="mathml">
			<xsl:apply-templates select="node()" mode="#current"/>
		</formula>
	</xsl:template>

<!--	<xsl:template match="dbk:info[not(local-name(..) = ('book', 'figure', 'article'))]/dbk:title | dbk:section/dbk:title | dbk:bibliodiv/dbk:title" priority="4" mode="hub2tei:dbk2tei">
		<head type="main">
			<xsl:apply-templates select="@*" mode="#current"/>
			<xsl:apply-templates select="if (parent::dbk:info or parent::dbk:bibliodiv) then ../../@xml:id else ../@xml:id" mode="#current"/>
			<xsl:apply-templates select="node()" mode="#current"/>
		</head>
	</xsl:template>
	-->
	<xsl:template match="tei:list/tei:item/@rend
	                    | tei:list/tei:item/tei:gloss/@rend
	                    |tei:list/tei:label/@rend
	                    | tei:cell/@rend[not(. = ('center', 'right'))]
	                    | tei:hi[not(node())]
	                    " mode="hub2tei:tidy"/>

  <xsl:template match="tei:p[parent::tei:item]" mode="hub2tei:tidy">
    <xsl:apply-templates select="node()" mode="#current"/>
  </xsl:template>

  <xsl:template match="tei:list[@type='bulleted']" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'unordered'"/>
      <xsl:apply-templates select="@* except (@type, @style), node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:graphic" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:apply-templates select="@*" mode="#current"/>
      <xsl:element name="desc">
        <xsl:value-of select="@url"/>
      </xsl:element>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="dbk:blockquote[@role= 'note']" mode="hub2tei:dbk2tei">
    <xsl:element name="note">
      <xsl:attribute name="rend" select="'note'"/>
      <xsl:apply-templates select="@* except @rend, node()" mode="#current"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="tei:hi[matches(@rend,'^[Ee]mphasis$|^[Ii]talic$')]
                              [@css:font-style='normal']" mode="hub2tei:tidy">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>

  <xsl:template match="tei:hi[matches(@rend,'^[Ss]trong$|^[Bb]old$')]
                              [@css:font-weight='normal']" mode="hub2tei:tidy">
    <xsl:apply-templates mode="#current"/>
  </xsl:template>

  <xsl:template match="tei:hi[matches(@rend,'^[Ss]trong$|^[Bb]old$')]
                              [@css:font-weight='bold']" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'Bold'"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:hi[matches(@rend,'^[Ee]mphasis$|^[Ii]talic$')]
                              [@css:font-style='italic']
                              [not(@css:font-weight='bold')]" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'Italics'"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:hi[@css:font-weight='bold']
                              [@css:font-style='italic']
                       | tei:hi[matches(@rend,'Bolditalic')]" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'BoldItalics'"/>
      <xsl:apply-templates mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="dbk:blockquote[@role= ('Accordion', 'NewContent')]" mode="hub2tei:dbk2tei">
    <xsl:element name="floatingText">
        <xsl:attribute name="rend" select="@role"/>
      <xsl:element name="body">
        <xsl:apply-templates select="@* except @role, node()" mode="#current"/>
      </xsl:element>
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="tei:p[1][parent::tei:body/parent::tei:floatingText[@rend = 'Accordion']]" mode="hub2tei:tidy">
    <xsl:element name="head">
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="dbk:blockquote" mode="hub2tei:dbk2tei" priority="-1">
    <xsl:apply-templates select="node()" mode="#current"/>
  </xsl:template>

  <xsl:template match="dbk:link/@linkend" mode="hub2tei:dbk2tei">
    <xsl:attribute name="target" select="."/>
  </xsl:template>

  <!--  create iframe for embedded videos -->
  <xsl:template match="dbk:para[dbk:mediaobject[not(matches(dbk:imageobject/dbk:imagedata/@fileref,'.*\.(png|jpe?g|tiff?|eps|wmf|gif)'))]]
                       | dbk:para[dbk:phrase[@role='video']]" mode="hub2tei:dbk2tei">
    <figure>
      <iframe xmlns="http://www.w3.org/1999/xhtml" src='{(dbk:phrase[@role="video"],descendant::dbk:imageobject/dbk:imagedata/@fileref)[1]}'
        width='640'
        height='360'
        frameborder='0'
        webkitallowfullscreen='true'
        mozallowfullscreen='true'
        allowfullscreen='true'>
      </iframe>
      <desc><xsl:value-of select="(dbk:phrase[@role='video'],descendant::dbk:imageobject/dbk:imagedata/@fileref)[1]"/></desc>
    </figure>
  </xsl:template>

<!--  <!-\-  use phrase role=video" with emdedded iframe code, requires Saxon PE/EE -\->
  <xsl:template match="dbk:para[dbk:phrase[@role='video']]" mode="hub2tei:dbk2tei">
    <xsl:apply-templates select="saxon:parse-html(dbk:phrase[@role='video'])/*:html/*:body/*" mode="#current"/>
  </xsl:template>

  <xsl:template match="*:iframe/@*[not(name()='src')]" mode="hub2tei:tidy">
    <!-\-<xsl:message select="name(), ., matches(name(),.)"></xsl:message>-\->
    <xsl:choose>
      <xsl:when test="matches(name(),.)">
        <xsl:attribute name="{name()}" select="'true'"/>
      </xsl:when>
    <xsl:otherwise>
      <xsl:copy/>
    </xsl:otherwise>
    </xsl:choose>
  </xsl:template>-->

  <xsl:template match="*:iframe" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>


  <xsl:key name="hub:linked-item-by-id" match="*[@xml:id]" use="@xml:id" />

  <xsl:template match="tei:ref[exists(key('hub:linked-item-by-id',replace(@target,'#','')))]" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'internal'"/>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:ref[matches(@target, '@')]" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="'mailto'"/>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:ref[matches(@target, 'https?:')]" mode="hub2tei:tidy" priority="2">
    <xsl:copy>
      <xsl:attribute name="rend" select="'external'"/>
      <xsl:apply-templates select="@*, node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:hi[not(@rend)][node()]" mode="hub2tei:tidy">
    <xsl:apply-templates select="node()" mode="#current"/>
  </xsl:template>

  <xsl:template match="tei:cell[tei:p[@rend]]" mode="hub2tei:tidy">
    <xsl:copy>
      <xsl:attribute name="rend" select="if (matches(tei:p[1]/@rend,'Tableheadermerged')) then 'Label' else tei:p[1]/@rend"/>
      <xsl:if test="matches(tei:p[1]/@rend,'Tableheadermerged')">
        <xsl:attribute name="cols" select="count(parent::tei:row/following-sibling::tei:row[1]/tei:cell)"/>
      </xsl:if>
      <xsl:apply-templates select="node()" mode="#current"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="tei:cell[not(descendant::text())]
                               [preceding-sibling::tei:cell[tei:p/@rend='Tableheadermerged']]" mode="hub2tei:tidy"/>

  <xsl:template match="tei:p[starts-with(./@rend,'Tableheader')][parent::tei:cell]" mode="hub2tei:tidy">
    <xsl:apply-templates select="node()" mode="#current"/>
  </xsl:template>

  <xsl:template match="tei:head[@xml:id]/*:anchor[@xml:id = parent::tei:head/@xml:id]"  mode="hub2tei:tidy"/>

  <xsl:template match="@css:*
                      | tei:seg[@type='tab']
                      | @annotations
                      | @type[not(./local-name() = 'name')]
                      | anchor
                      | tei:div/@rend[not(matches(.,'Accordion|Note|NewContent'))]
                      | tei:item/@n
                      " mode="hub2tei:tidy"/>

</xsl:stylesheet>
