<p:declare-step 
  xmlns:p="http://www.w3.org/ns/xproc"
  xmlns:c="http://www.w3.org/ns/xproc-step"
  xmlns:docx2hub="http://transpect.io/docx2hub"
  xmlns:hub2htm="http://transpect.io/hub2htm"
  xmlns:tr="http://transpect.io"
  xmlns:cx="http://xmlcalabash.com/ns/extensions" 
  name="docx2epub"
  version="1.0">
  
  <p:input port="conf" primary="true">
    <p:documentation xmlns="http://www.w3.org/1999/xhtml">
      <p>See the section on 
        <a href="https://subversion.le-tex.de/common/transpect-demo/content/le-tex/setup-manual/en/out/xhtml/transpect-setup.xhtml#sec-cascade">configuration clades.</a>
      </p>
      <p>The converter works also without clade-based configuration. If you store your content in an arbitrary directory (e.g,
        /my/dir/idml/mycontent.idml), you may put content-specific configuration overrides into /my/dir/css, /my/dir/evolve-hub,
        etc.</p>
    </p:documentation>
    <p:document href="http://this.transpect.io/conf/transpect-conf.xml"/>
  </p:input>
  
  <p:input port="schema" primary="false">
    <p:documentation>Expects the Hub RelaxNG XML schema</p:documentation>
    <p:document href="http://www.le-tex.de/resource/schema/hub/1.1/hub.rng"/>
  </p:input>
  
  <p:output port="hub">
    <p:pipe port="result" step="docx2hub"/>
  </p:output>
   
  <p:output port="paths">
    <p:pipe port="result" step="paths"/>
  </p:output>
  <p:serialization port="paths" indent="true"/>

  <p:output port="tei">
    <p:pipe port="tei" step="flat2epub"/>
  </p:output>
  
  <p:output port="html" sequence="true">
    <p:pipe port="html" step="flat2epub"/>
  </p:output>  
 	<p:serialization port="html" omit-xml-declaration="true" indent="false"/>
	
  <p:output port="htmlreport">
    <p:pipe port="htmlreport" step="flat2epub"/>
  </p:output>
  <p:serialization port="htmlreport" omit-xml-declaration="false"/>
  
  <p:option name="docxfile" required="true" />
  <p:option name="hub-version" select="'1.2'"/>
  
  <p:option name="clades" select="''"/> 
  
  <p:option name="debug" select="'yes'"/> 
  <p:option name="debug-dir-uri" select="'debug'"/>
  <p:option name="status-dir-uri" select="'status'"/>
  
  <p:option name="progress" required="false" select="'yes'"/>
  <p:option name="check" required="false" select="'yes'"/>
  
  <p:option name="interface-language" required="false" select="'en'"/>
  <p:option name="epub-check-http-resources" required="false" select="'_default_'">
    <p:documentation>true/false override for conf param with same name. Made this configurable separately
    because link checking takes so much time.</p:documentation>
  </p:option>
  
  <p:option name="ub-id" required="false" select="''">
    <p:documentation>If this param is given, a zip package will be created by this name 
    	and including differently named files, the images as well as relative paths for images and CSS files in the created HTML.</p:documentation>
  </p:option>

  <p:import href="http://xmlcalabash.com/extension/steps/library-1.0.xpl"/>
  
  <p:import href="http://transpect.io/calabash-extensions/unzip-extension/unzip-declaration.xpl"/>
  <p:import href="http://transpect.io/docx2hub/xpl/docx2hub.xpl"/>
  <p:import href="http://transpect.io/hub2html/xpl/hub2html.xpl"/>
  <p:import href="http://transpect.io/cascade/xpl/paths.xpl"/>
  <p:import href="flat-hub2epub.xpl"/>
  <p:import href="http://transpect.io/xproc-util/simple-progress-msg/xpl/simple-progress-msg.xpl"/>
  <p:import href="http://transpect.io/xproc-util/store-debug/xpl/store-debug.xpl" />
  <p:import href="http://transpect.io/htmlreports/xpl/validate-with-schematron.xpl"/>
  <p:import href="http://transpect.io/htmlreports/xpl/errorPI2svrl.xpl"/>
  <p:import href="http://transpect.io/calabash-extensions/rng-extension/xpl/rng-validate-to-PI.xpl"/>
  
    <cx:message>
<!--  	<p:input port="source"><p:empty/></p:input>-->
    <p:with-option name="message" select="'### xml2idml: now unzipping IDML template'"/>
  </cx:message> 
	
  <p:string-replace name="start-msg-replace" match="file">
    <p:with-option name="replace" select="concat('''', replace($docxfile, '^.+/', ''), '''')"/>
    <p:input port="source">
      <p:inline>
        <c:messages>
          <c:message xml:lang="en">////////// Starting docx2epub conversion of <file/> \\\\\\\\\\</c:message>
          <c:message xml:lang="de">////////// Beginne docx2epub-Konvertierung von <file/> \\\\\\\\\\</c:message>
        </c:messages>
      </p:inline>
    </p:input>    
  </p:string-replace>
  
  <tr:simple-progress-msg name="start-msg" file="docx2epub-start.txt">
    <p:input port="msgs">
      <p:pipe port="result" step="start-msg-replace"/>
    </p:input>
    <p:with-option name="status-dir-uri" select="$status-dir-uri"/>
  </tr:simple-progress-msg>
  
  <tr:paths name="orig-paths" cx:depends-on="start-msg">
    <p:with-option name="pipeline" select="'docx2epub.xpl'"/>
    <p:with-option name="clades" select="$clades"/>
    <p:with-option name="file" select="$docxfile"/>
    <p:with-option name="interface-language" select="$interface-language"/>
    <p:with-option name="debug" select="$debug"/>
    <p:with-option name="debug-dir-uri" select="$debug-dir-uri"/>
    <p:with-option name="status-dir-uri" select="$status-dir-uri"/>
    <p:with-option name="progress" select="$progress"/>
    <p:input port="conf">
      <p:pipe port="conf" step="docx2epub"/>
    </p:input>
    <p:input port="stylesheet">
      <p:document href="http://this.transpect.io/a9s/common/xsl/paths.xsl"/>
    </p:input>
  </tr:paths>
  
  
	<p:add-attribute match="/*" attribute-name="value" name="ub-id">
    <p:with-option name="attribute-value" select="$ub-id"/>
    <p:input port="source">
      <p:inline>
        <c:param name="ub-id"/>
      </p:inline>
    </p:input>
  </p:add-attribute>

  <p:insert position="last-child" match="/*" name="paths">
    <p:input port="insertion">
      <p:pipe port="result" step="ub-id"/>
    </p:input>
    <p:input port="source">
      <p:pipe port="result" step="orig-paths"/>
    </p:input>
  </p:insert>
	
	<tr:store-debug pipeline-step="cascade/custom-paths">
    <p:with-option name="active" select="$debug" />
    <p:with-option name="base-uri" select="$debug-dir-uri" />
  </tr:store-debug>
	
  <p:sink/>
  
  <docx2hub:convert name="docx2hub">
    <p:with-option name="debug" select="$debug"/>
    <p:with-option name="debug-dir-uri" select="$debug-dir-uri"/>
    <p:with-option name="status-dir-uri" select="$status-dir-uri"/>
    <p:with-option name="docx" select="/c:param-set/c:param[@name eq 'file']/@value">
      <p:pipe port="result" step="paths"/>
    </p:with-option>
    <p:with-option name="hub-version" select="$hub-version"/>
    <p:with-option name="srcpaths" select="'yes'"/>
  </docx2hub:convert>
    
  <tr:flat2epub name="flat2epub" >
    <p:input port="paths">
      <p:pipe port="result" step="paths"/>
    </p:input>
    <p:input port="reports">
      <p:pipe port="report" step="docx2hub"/>
    </p:input>
    <p:with-option name="debug" select="$debug"/>
    <p:with-option name="debug-dir-uri" select="$debug-dir-uri"/>
    <p:with-option name="interface-language" select="$interface-language"/>
    <p:with-option name="status-dir-uri" select="$status-dir-uri"/>
  	<p:with-option name="ub-id" select="$ub-id"/>
  </tr:flat2epub>
   
  <p:string-replace name="success-msg-replace" match="file">
    <p:with-option name="replace" select="concat('''', replace($docxfile, '^.+/', ''), '''')"/>
    <p:input port="source">
      <p:inline>
        <c:messages>
          <c:message xml:lang="en">\\\\\\\\\\ Successfully finished docx2epub conversion of <file/> //////////</c:message>
          <c:message xml:lang="de">\\\\\\\\\\ docx2epub-Konvertierung von <file/> erfolgreich abgeschlossen //////////</c:message>
        </c:messages>
      </p:inline>
    </p:input>    
  </p:string-replace>
  
  <tr:simple-progress-msg name="success-msg" file="docx2epub-success.txt" cx:depends-on="flat2epub">
    <p:input port="msgs">
      <p:pipe port="result" step="success-msg-replace"/>
    </p:input>
    <p:with-option name="status-dir-uri" select="$status-dir-uri"/>
  </tr:simple-progress-msg>
  
  <p:sink/>

</p:declare-step>