<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:tr="http://transpect.io"
  exclude-result-prefixes="xs"
  version="2.0">
  <xsl:import href="http://transpect.io/cascade/xsl/paths.xsl"/>
  <xsl:param name="filenames" as="xs:string?"/>
  <!-- that param is for filenames like images -->
  <xsl:variable name="regex" select="'^.+$'" as="xs:string"/>
  
  <xsl:function name="tr:parse-file-name" as="attribute(*)*">
    <xsl:param name="filename" as="xs:string?"/>
    <xsl:variable name="basename" select="tr:basename($filename)" as="xs:string"/>
    <xsl:variable name="ext" select="tr:ext($filename)" as="xs:string"/>
    <xsl:attribute name="ext" select="$ext"/>
    <xsl:attribute name="base" select="$basename"/>
  
    <xsl:analyze-string select="$basename" regex="{$regex}">
      <xsl:matching-substring>
        <xsl:choose>
          <xsl:when test="not($ext = ('docx', 'xml', 'idml', 'png', 'jpg', 'tif', 'epub', 'mobi', 'report.xhtml', 'html'))"/>
          <!--<xsl:when test="matches(regex-group(1),'^KdS$')">
            <xsl:attribute name="publisher" select="'GSK'"/>
            <xsl:attribute name="imprint" select="regex-group(1)"/>
          	<xsl:attribute name="volume" select="regex-group(3)"/>
          </xsl:when>-->
          <xsl:otherwise>
          	<xsl:attribute name="publisher" select="regex-group(1)"/>
          	<xsl:attribute name="imprint" select="regex-group(2)"/>
          	<xsl:attribute name="volume" select="regex-group(3)"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:matching-substring>
      <xsl:non-matching-substring>
        <xsl:attribute name="ms" select="."/>
      </xsl:non-matching-substring>
    </xsl:analyze-string>
  </xsl:function>
    
  
  <xsl:function name="tr:target-subdir" as="xs:string">
    <xsl:param name="content" as="element(tr:content)"/>
    <xsl:choose>
      <xsl:when test="$content/@ext = ('png', 'PNG', 'jpg', 'JPG', 'jpeg', 'JPEG')">
        <xsl:sequence select="if ($content/@base[matches(., '_COVER$')]) then 'images/cover' else 'images'"/>
      </xsl:when>
      <xsl:when test="$content/@ext[matches(., '(epub|mobi)')]">
        <xsl:sequence select="'epub'"/>
      </xsl:when>
      <xsl:when test="$content/@ext = 'xml'">
        <xsl:sequence select="'tei'"/>
      </xsl:when>
      <xsl:when test="$content/@ext = 'html'">
        <xsl:sequence select="'html'"/>
      </xsl:when>
    	<xsl:when test="$content/@ext = 'report.xhtml'">
        <xsl:sequence select="'report'"/>
      </xsl:when>
    	<xsl:when test="$content/@ext = 'idml'">
        <xsl:sequence select="'idml'"/>
      </xsl:when>
    	<xsl:when test="$content/@ext = 'docx'">
        <xsl:sequence select="'docx'"/>
      </xsl:when>
    </xsl:choose>
  </xsl:function>
	
	
</xsl:stylesheet>